/*------------------------------------------------------------------------
--                                                                      --
--                     H E L I O S   K E R N E L                        --
--                     -------------------------                        --
--                                                                      --
--             Copyright (C) 1987, Perihelion Software Ltd.             --
--                        All Rights Reserved.                          --
--                                                                      --
-- device.h								--
--                                                                      --
--	Machine independant device interface.				--
--	This is not intended to be a full-blown device system, no	--
-- 	provision is made here for multiple clients, or for making	--
--	the device accessible to more than one client. Rather this is	--
--	designed to encapsulate only the hardware specific parts of a	--
--	device. Sharing the device between clients, and converting to	--
--	a GSP interface is the job of the device's client which should	--
--	generally be a Helios server of some sort.			--
--                                                                      --
--	Author:  NHG 8/8/88						--
--                                                                      --
------------------------------------------------------------------------*/
/* SccsId: @(#)device.h	1.5	5/4/89 Copyright (C) 1987, Perihelion Software Ltd.	*/

#ifndef __device_h
#define __device_h

#ifndef __helios_h
#include <helios.h>
#endif

#include <syslib.h>
#include <queue.h>

typedef struct Device {
	word		Type;		/* module type = T.Device	*/
	word		Size;		/* size of device in bytes	*/
	char		Name[32];	/* device name			*/
	word		Id;		/* not used 			*/
	word		Version;	/* version number of this device*/
	RPTR		Open;		/* offset of open routine	*/
} Device;

/* This is intended to be an initial sub-structure of a larger device	*/
/* specific control block.						*/
typedef struct DCB {
	Device		*Device;	/* pointer to Device struct 	*/
	WordFnPtr	Operate;	/* action entry point		*/
	WordFnPtr	Close;		/* close device routine		*/
	Stream		*Code;		/* open code stream 		*/
	List		Requests;	/* pending request queue	*/
	List		Replies;	/* completed request queue	*/
} DCB;

/* The form of this structure is provisional, the exact form is to be	*/
/* determined by consultation with ParsyTec.				*/
/* Again this is intended to be an initial sub-structure of something	*/
/* larger.								*/
/* In general there will be at least one more layer on top of this to	*/
/* define things like generic disc, serial and parallel devices.	*/

typedef struct DevReq
{
	Node		Node;		/* queueing node		*/
	word		Request;	/* request type			*/
	word		Result;		/* error code/result		*/
	VoidFnPtr	Action;		/* termination action		*/
	word		SubDevice;	/* internal sub-device		*/
	word		Timeout;	/* timeout, or -1 == forever	*/
} DevReq;


typedef struct DiscReq
{
	DevReq		DevReq;		/* standard device request	*/
	word		Pos;		/* tfr start position		*/
	word		Size;		/* tfr size			*/
	void		*Buf;		/* buffer			*/
	word		Actual;		/* data actually transferred	*/
	Semaphore	WaitLock;	/* client waits here		*/
} DiscReq;

/* macro to call device Operate entry					*/
#define Operate(dcb,req)	(dcb->Operate)(dcb,req)

extern DCB    *OpenDevice(string name,void *info);
extern word   CloseDevice(DCB *dcb);

#endif

/* -- End of device.h */
