
/* posix.h : Posix Library function definitions.			*/

/* These are function prototypes for those routines which do not depend	*/
/* on any other header file. Including this keeps the compiler happy.	*/

#ifndef __posix_h
#define __posix_h

#ifndef __types_h
#include <sys/types.h>
#endif

extern int 	errno;
extern int	oserr;

extern char 	**environ;

extern char 	**_posix_init(void);

extern int 	open(char *name, int oflag, ... );
extern int 	creat(char *path, mode_t mode);
extern int 	umask(mode_t cmask);
extern int 	link(char *path1, char *path2);
extern int 	mkdir(char *path, mode_t mode);
extern int 	mkfifo(char *path, mode_t mode);
extern int 	unlink(char *path);
extern int 	rmdir(char *path);
extern int 	rename(const char *old, const char *new);
extern int 	access(char *path, int amode);
extern int 	chmod(char *path, mode_t mode);
extern int 	chown(char *path, uid_t owner, uid_t group);
extern int 	pathconf(char *path, int name);
extern int 	fpathconf(int fd, int name);
extern int 	pipe(int fildes[2]);
extern int 	dup(int fd);
extern int 	dup2(int fd,int fd2);
extern int 	close(int fd);
extern int 	read(int fd, char *buf, unsigned nbyte);
extern int 	write(int fd, char *buf, int nbyte);
extern int 	fcntl(int fd, int cmd, ... );
extern off_t 	lseek(int fd, off_t offset, int whence);

extern int 	chdir(char *path);
extern char 	*getcwd(char *buf, int size);

extern int 	getpid(void);
extern int 	getppid(void);
extern uid_t 	getuid(void);
extern uid_t 	geteuid(void);
extern uid_t 	getgid(void);
extern uid_t 	getegid(void);
extern uid_t 	setuid(uid_t uid);
extern uid_t 	setgid(uid_t uid);
extern int 	getgroups(int setsize, uid_t *list);
extern char	*getlogin(void);
extern char	*cuserid(char *s);
extern int 	getprgrp(void);
extern int 	setprgrp(void);
extern int 	jcsetpgrp(int pgrp);
extern time_t 	time(time_t *tloc);
extern char 	*getenv(const char *name);
extern char 	*ctermid(char *s);
extern char	*ttyname(int fd);
extern int 	isatty(int fd);
extern int 	sysconf(int name);

extern int 	vfork(void);
extern int 	execl(char *path,...);
extern int 	execv(char *path,char **argv);
extern int 	execle(char *path,...);
extern int 	execlp(char *file,...);
extern int 	execvp(char *file, char **argv);
extern int 	execve(char *name, char **argv, char **envv);
extern void 	_exit(int code);
extern int	wait(int *statloc);

/* The following are Helios extensions to map from Posix to Helios objects */

#ifdef __syslib_h

extern Stream 	*fdstream(int fd);
extern Object 	*cdobj(void);
extern int 	sopen(Stream *s);
extern Environ  *getenviron(void);

#endif

/* Helios extensions to POSIX */

#define		PE_BLOCK	1
#define		PE_UNBLOCK	2
#define		PE_SETMASK	3

#define		PE_RemExecute	1	/* All Execute requests may be sent */
					/* To the TFM if this bit is set in */
					/* pflagword			    */

extern int _posixflags(int how, int set );

#endif /* __posix_h */
