
/* signal.h: ANSI draft (Oct 86) library header, section 4.7 */
/* Copyright (C) A.C. Norman and A. Mycroft */
/* version 0.01 */

#ifndef __signal_h
#define __signal_h

typedef unsigned long	sigset_t;

struct sigaction {
	void		(*sa_handler)();
	sigset_t	sa_mask;
	int		sa_flags;
};

extern void _ignore_signal_handler(int);
extern void _default_signal_handler(int);
extern void _error_signal_marker(int);

#define SIG_IGN _ignore_signal_handler
#define SIG_DFL _default_signal_handler
#define SIG_ERR _error_signal_marker

#define SIGABRT 1   /* abort                         */
#define SIGFPE  2   /* arithmetic exception          */
#define SIGILL  3   /* illegal instruction           */
#define SIGINT  4   /* attention request from user   */
#define SIGSEGV 5   /* bad memory access             */
#define SIGTERM 6   /* termination request           */
#define SIGSTAK 7   /* stack overflow                */

#define SIGALRM 8   /* alarm/timeout signal		*/
#define SIGHUP  9   /* hangup				*/
#define SIGPIPE 10  /* pipe signal			*/
#define SIGQUIT 11 
#define SIGTRAP 12
#define SIGUSR1 13
#define SIGUSR2 14

#define SIGKILL 31  /* termination signal		*/

/* codes for sigprocmask */
#define SIG_BLOCK	1
#define SIG_UNBLOCK	2
#define SIG_SETMASK	3


extern int kill(int pid, int sig);
extern int siginitset(sigset_t *set);
extern int sigfillset(sigset_t *set);
extern int sigaddset(sigset_t *set, int sig);
extern int sigdelset(sigset_t *set, int sig);
extern int sigismember(sigset_t *set, int sig);
extern int sigaction(int sig, struct sigaction *act, struct sigaction *oact);
extern int sigprocmask( int how, sigset_t *set, sigset_t *oset);
extern int sigpending(sigset_t *set);
extern int sigsuspend(sigset_t *set);
extern unsigned int alarm(unsigned int sec);
extern int pause(void);
extern unsigned int sleep(unsigned int seconds);
extern void (*signal (int sig, void (*func)(int)))(int);
extern int raise(int sig);

#endif

/* end of signal.h */
