/*------------------------------------------------------------------------
--                                                                      --
--                     H E L I O S   N U C L E U S                      --
--                     ---------------------------                      --
--                                                                      --
--             Copyright (C) 1987, Perihelion Software Ltd.             --
--                        All Rights Reserved.                          --
--                                                                      --
-- syslib.h								--
--                                                                      --
--	System library header.						--	
--                                                                      --
--	Author:  NHG 16/8/87						--
--                                                                      --
------------------------------------------------------------------------*/
/* SccsId: @(#)syslib.h	1.13	13/3/89 Copyright (C) 1987, Perihelion Software Ltd.	*/

#ifndef __syslib_h
#define __syslib_h

#ifndef __helios_h
#include <helios.h>
#endif

#include <queue.h>

#ifndef __sem_h
#include <sem.h>
#endif
#ifndef __protect_h
#include <protect.h>
#endif
#ifndef __task_h
#include <task.h>
#endif
#ifndef __message_h
typedef word		Port;		/* message port			*/
#endif

/*----------------------------------------------------------------------*/
/* Date stamp 								*/
/*----------------------------------------------------------------------*/

typedef unsigned int Date;		/* in seconds since 1/1/70	*/

/* common set of dates used by most objects */
typedef struct DateSet {
	Date		Creation;	/* time object was created	*/
	Date		Access;		/* time object last accessed	*/
	Date		Modified;	/* time object last modified	*/
} DateSet;

/*----------------------------------------------------------------------*/
/* Passive object description structure 				*/
/*----------------------------------------------------------------------*/

typedef struct Object {
        Node		Node;           /* link in housekeeping list    */
        word            Type;           /* object type                  */
        word            Flags;          /* control flag bits            */
        word            Result2;        /* second result or error       */
	word		FnMod;		/* function code modifier	*/
	word		Timeout;	/* seed for timeouts		*/
        Port            Reply;          /* port for object replies      */
        Capability      Access;         /* access capability            */
        byte            Name[Variable]; /* object pathname	        */
} Object;

/*----------------------------------------------------------------------*/
/* Active object structure 						*/
/*----------------------------------------------------------------------*/

typedef struct Stream {
        Node		Node;           /* link in housekeeping list    */
        word            Type;           /* stream type                  */
        word            Flags;          /* flag bits                    */
        word            Result2;        /* second result or error       */
	word		FnMod;		/* function code modifier	*/
	word		Timeout;	/* seed for timeouts		*/
        Port            Reply;          /* reply port                   */
        Capability      Access;         /* access capability            */
        word            Pos;            /* current object position      */
        Port            Server;         /* server message port          */
	Semaphore	Mutex;		/* protect against multi-access */
        char            Name[Variable]; /* full object path name        */
} Stream;

/*----------------------------------------------------------------------*/
/* Directory Entry structure						*/
/*----------------------------------------------------------------------*/

typedef struct DirEntry {
	word		Type;		/* entry type			*/
	word		Flags;		/* flag bits			*/
	Matrix		Matrix;		/* access matrix		*/
	char		Name[32];	/* entry name			*/
} DirEntry;

/*----------------------------------------------------------------------*/
/* Generic Object Info structure					*/
/*----------------------------------------------------------------------*/

typedef struct ObjInfo {
	DirEntry	DirEntry;	/* re-iteration of common info	*/
	word		Account;	/* servers accounting info	*/
	word		Size;		/* file size in bytes		*/
	DateSet		Dates;		/* object dates			*/
} ObjInfo;

/*----------------------------------------------------------------------*/
/* Link Info returned from ObjInfo					*/
/*----------------------------------------------------------------------*/

typedef struct Link_Info {
	DirEntry	DirEntry;	/* re-iteration of common info	*/
	Capability	Cap;		/* capability for target	*/
	char		Name[Variable];	/* full target path name	*/
} Link_Info;

/*----------------------------------------------------------------------*/
/* Task Creation information						*/
/*----------------------------------------------------------------------*/

typedef struct TaskInfo {
	RPTR		Name;		/* name of program file		*/
	Capability	Cap;		/* capability for it		*/
	Matrix		Matrix;		/* matrix for new task		*/
} TaskInfo;

/*----------------------------------------------------------------------*/
/* Environment description structure					*/
/*----------------------------------------------------------------------*/

typedef struct Environ {
	string		*Argv;		/* argument strings		*/
	string		*Envv;		/* environment strings		*/
	Object		**Objv;		/* objects to be passed		*/
	Stream		**Strv;		/* streams to be passed		*/
} Environ;

/*----------------------------------------------------------------------*/
/* Marshalled environment descriptors					*/
/*----------------------------------------------------------------------*/

typedef struct ObjDesc {
	Capability	Cap;		/* capability for object	*/
	char		Name[Variable];	/* object path name		*/
} ObjDesc;

typedef struct StrDesc {
	word		Mode;		/* mode stream was opened in	*/
	word		Pos;		/* position to start at		*/
	Capability	Cap;		/* capability for stream	*/
	char		Name[Variable];	/* name of stream object	*/
} StrDesc;

/*----------------------------------------------------------------------*/
/* Program Load information						*/
/*----------------------------------------------------------------------*/

typedef struct LoadInfo {
	Capability	Cap;		/* access permission		*/
	Matrix		Matrix;		/* new access matrix		*/
	word		Pos;		/* position of image in file	*/
	char		Name[Variable];	/* file name			*/
} LoadInfo;

/*----------------------------------------------------------------------*/
/* Modes for Open							*/
/*----------------------------------------------------------------------*/

#define O_Mask		0xff		/* mask for access bits		*/
#define O_ReadOnly	1		/* open for reading only	*/
#define O_WriteOnly	2		/* open for writing only	*/
#define O_ReadWrite	3		/* open for reading & writing	*/
#define O_Execute	4		/* open for execution		*/
#define O_Create	0x100		/* create if does not exist	*/
#define O_Exclusive	0x200		/* get exclusive access		*/
#define O_Truncate	0x300		/* truncate if already exists	*/
#define O_Block		0x400		/* Wait for connect (Fifos)	*/
#define O_Sync		0x800		/* synchonous tfrs (Fifos)	*/

/*----------------------------------------------------------------------*/
/* Modes for Seek							*/
/*----------------------------------------------------------------------*/

#define S_Beginning	0		/* relative to start of file	*/
#define S_Relative	1		/* relative to current pos	*/
#define S_End		2		/* relative to end of file	*/
#define S_Last		3		/* relative to last operation	*/

/*----------------------------------------------------------------------*/
/* Flag Bits								*/
/*----------------------------------------------------------------------*/

/* First column of comments indicates whose responsibility the state 	*/
/* of the bit is: S = Server, L = Syslib.				*/

#define Flags_Mode		0x0000000f	/* L copy of open mode		*/
#define Flags_More		0x00000010	/* S More info available	*/
#define Flags_Seekable		0x00000020	/* S stream is seekable		*/
#define Flags_Remote		0x00000040	/* L server is non-local	*/
#define Flags_StripName		0x00000080	/* S names are stripped before pass on */
#define Flags_CacheName 	0x00000100	/* S name is cached		*/
#define Flags_LinkName  	0x00000200	/* S name is for h/w link	*/
#define Flags_PStream		0x00000400	/* L set for PseudoStreams	*/
#define Flags_ResetContext 	0x00000800	/* S set for remote servers	*/
#define Flags_Pipe		0x00001000	/* S set for pipes		*/
#define Flags_CloseOnSend	0x00002000	/* S close in SendEnv		*/
#define Flags_OpenOnGet		0x00004000	/* S open in GetEnv		*/

#define Flags_Interactive 	0x00010000	/* S if stream is interactive	*/
#define Flags_MSdos	  	0x00020000	/* S MSdos format files		*/

#define Flags_CloseMask 0xe0000000	/*   mask for following flags	*/
#define Flags_CloseShift	29	/*   shift value		*/
#define Flags_Closeable	0x20000000	/* S set if needs closing	*/
#define Flags_Server	0x40000000	/* S set if served stream	*/
#define Flags_Stream	0x80000000	/* L set if stream structure	*/

#define closebits_(x)    ((((UWORD)(x))>>Flags_CloseShift)&0x7)

/*----------------------------------------------------------------------*/
/* Codes for TaskData							*/
/*----------------------------------------------------------------------*/

#define TD_Set		1		/* set rather than read data	*/
#define TD_Program	2		/* base of program code		*/
#define TD_Port		4		/* own port			*/
#define TD_Parent	6		/* parents port			*/
#define TD_HeapBase	8		/* base of heap			*/
#define TD_Flags	10		/* Task flags			*/
#define TD_Pool		12		/* task memory pool		*/

/*----------------------------------------------------------------------*/
/* Program status codes							*/
/*----------------------------------------------------------------------*/

#define PS_Terminate	0x100		/* wait for program to stop	*/
#define PS_Suspend	0x200		/* wait for program pause	*/
#define PS_Restart	0x400		/* wait for program restart	*/
#define PS_Status	0x800		/* update for all components	*/

/*----------------------------------------------------------------------*/
/* Public Variables							*/
/*----------------------------------------------------------------------*/

extern Task *MyTask;			/* pointer to task struct	*/

/*----------------------------------------------------------------------*/
/* Function Prototypes							*/
/*----------------------------------------------------------------------*/

/* Object operations */
extern Stream *	Open		(Object *context, string name, word mode);
extern Object *	Locate		(Object *context, string name);
extern Object *	Create		(Object *context, string name, word type, word infosize, byte *info);
extern word	ObjectInfo	(Object *context, string name, byte *info);
extern word	Link		(Object *context, string name, Object *object);
extern word	SetDate		(Object *context, string name, DateSet *dates);
extern word	Protect		(Object *context, string name, Matrix matrix);
extern word	Delete		(Object *context, string name);
extern word	Rename		(Object *context, string name, string newname);
extern word	Refine		(Object *context,	       AccMask mask);
extern word	ServerInfo	(Object *context,              byte *info);

/* Stream operations */
extern word	Read		(Stream *stream, byte *buf, word size, word timeout);
extern word	Write		(Stream *stream, byte *buf, word size, word timeout);
extern word	Seek		(Stream *stream, word whence, word pos);
extern word	GetFileSize	(Stream *stream);
extern word	SetFileSize	(Stream *stream, word size);
extern void	ReOpen		(Stream *stream);
extern word	GetInfo		(Stream *stream, byte *info);
extern word	SetInfo		(Stream *stream, byte *info, word infosize);
extern Port	EnableEvents	(Stream *stream, word mask);

/* Pipe operations */
extern word	GrabPipe	(Stream *stream, Port *ports);
extern word	UnGrabPipe	(Stream *stream);

/* Program Loading and Execution */
extern Object *	(Load)		(Object *loader, Object *image);
extern Object *	Execute		(Object *procman, Object *image);
extern word	SendSignal	(Stream *task, word signum);
extern word	InitProgramInfo (Stream *task, word mask);
extern word	GetProgramInfo	(Stream *task, word *status_vec, word timeout);
extern word	DefaultException(word signum, void *data);

/* Heap management */
extern void *	Malloc		(word size);
extern word 	Free		(void *block);

/* Polymorphic operations */
extern word 	(Close)		(Stream *stream);
extern word	(Result2)	(Object *object);
extern word	(Abort)		(Object *object);

#ifndef in_syslib
/* These macros prevent compiler warnings */
#define Close(x)	Close((Stream *)(x))
#define Result2(x)	Result2((Object *)(x))
#define Abort(x)	Abort((Object *)(x))
#define Load(x,y)	Load(x,(Object *)(y))
#endif

/* Miscellaneous */
extern Date 	GetDate		(void);
extern void 	Exit		(word code);
extern void 	TidyUp		(void);
extern Stream *	NewStream	(string path, Capability *cap, word mode);
extern Stream *	PseudoStream	(Object *object, word mode);
extern Object *	NewObject	(string path, Capability *cap);
extern Object * CopyObject	(Object *object);
extern void 	SendIOC		(MCB *mcb);
extern word 	MachineName	(byte *name);
extern word	Alarm		(word seconds);
extern word	TaskData	(word act, void *result);

/* Port management */
extern Port 	NewPort		(void);
extern word	FreePort	(Port port);

/* Parameter Marshalling */
extern void 	InitMCB		(MCB *mcb, byte flags, Port dest, Port reply, word fnrc);
extern void 	MarshalString	(MCB *mcb, string str);
extern void 	MarshalWord	(MCB *mcb, word value);
extern void 	MarshalOffset	(MCB *mcb);
extern void 	MarshalCap	(MCB *mcb, Capability *cap);
extern void 	MarshalDate	(MCB *mcb, Date date);
extern void 	MarshalCommon	(MCB *mcb, Object *object, string name);
extern void 	MarshalData	(MCB *mcb, word size, byte *data);
extern void 	MarshalStream	(MCB *mcb, Stream *stream);
extern void 	MarshalObject	(MCB *mcb, Object *object);

/* Matrix & Capability En/Decoding */
extern Matrix 	EncodeMatrix	(string str, word type);
extern string 	DecodeMask	(string str, AccMask mask, string str);
extern void 	DecodeMatrix	(string str, Matrix matrix, word type);
extern string 	getbitchars	(word type);
extern char	*EncodeCapability(char *s, Capability *cap);
extern char	*DecodeCapability(char *s,Capability *cap);

/* Environment manipulation */
extern word 	SendEnv		(Port dest, Environ *env);
extern word 	GetEnv		(Port port, Environ *env);

/* File name manipulation */
extern int 	splitname	(char *prefix, char ch, char *str);

/* This is really in the kernel */
extern void 	Delay		(word micros);

/* For compatability... */
extern Object *cdobj(void);	/* actually in POSIX library */
#define CurrentDir cdobj()

#endif

/* -- End of syslib.h */
